<?php

namespace Drupal\gotem\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\gotem\Event\ContentModerationEvent;
use Drupal\gotem\OpenAIModerationClient;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\node\Entity\Node;
use Drupal\Core\Entity\EntityStorageException;
use Drupal\gotem\Exception\ModerationException;

/**
 * Class ContentModerationSubscriber.
 *
 * @package Drupal\gotem\EventSubscriber
 */
class ContentModerationSubscriber implements EventSubscriberInterface {

  protected $openAIModerationClient;
  protected $currentUser;
  protected $messenger;
  protected $logger;

  /**
   * Constructs a new ContentModerationSubscriber object.
   */
  public function __construct(
    OpenAIModerationClient $openai_moderation_client,
    AccountProxyInterface $current_user,
    MessengerInterface $messenger,
    LoggerChannelFactoryInterface $logger_factory
  ) {
    $this->openAIModerationClient = $openai_moderation_client;
    $this->currentUser = $current_user;
    $this->messenger = $messenger;
    $this->logger = $logger_factory->get('gotem');
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      ContentModerationEvent::class => 'onContentModeration',
    ];
  }

  /**
   * Responds to the content moderation event.
   *
   * @param \Drupal\gotem\Event\ContentModerationEvent $event
   *   The event object.
   */
  public function onContentModeration(ContentModerationEvent $event) {
    $entity = $event->getEntity();

    // Only proceed if the entity is a node.
    if ($entity instanceof \Drupal\node\NodeInterface) {
      $this->logger->info('ContentModerationSubscriber fired for node of type @type.', ['@type' => $entity->bundle()]);

      // Get text fields to moderate.
      $fields_to_check = ['title', 'body'];

      $text_to_check = '';
      foreach ($fields_to_check as $field_name) {
        if ($entity->hasField($field_name) && !$entity->get($field_name)->isEmpty()) {
          $text = $entity->get($field_name)->value;
          $text_to_check .= $text . ' ';
          $this->logger->debug('Field @field value: @value', ['@field' => $field_name, '@value' => $text]);
        }
      }

      $this->logger->info('Text to be moderated: @text', ['@text' => $text_to_check]);

      // Use the OpenAI client to moderate the text.
      $result = $this->openAIModerationClient->moderateText($text_to_check);

      $this->logger->info('Moderation result: @result', ['@result' => print_r($result, TRUE)]);

      if ($result && !empty($result['flagged'])) {
        $node = \Drupal::entityTypeManager()->getStorage('node')->create(array(
          'type'        => 'moderation_flag',
          'title'       => 'Flagged Content',
          'field_user'  => $this->currentUser->id(),
          'field_json_result' => json_encode($result),
        ));

        $node->save();
        $this->messenger->addError('Content contains disallowed material and cannot be saved.');
        $this->logger->warning('Content moderation failed for entity ID @id.', ['@id' => $entity->id()]);

        //redirect to page home 

        // prevent the entity from being saved
        $event->stopPropagation();
        $response = new RedirectResponse('/moderation-failed');
        $response->send();
      }
    }
  }

}
